import random

from char_collection.collect import CharacterSequence


class CollectPassword(CharacterSequence):
    """
    A class generating a random password of different characters.
    """

    @staticmethod
    def _collect(length, collect_chars):
        length = int(length)
        if length == 6:
            random.shuffle(collect_chars)
            return ''.join(collect_chars[:6])
        elif 6 < length <= 20:
            random.shuffle(collect_chars)
            return ''.join(collect_chars[:length])
        else:
            raise ValueError('password length can be from 6 to 20 characters')


class CollectPasswordNotSpecialSymbol(CollectPassword):
    """
    A class that generates a password without special characters
    """

    @property
    def _collect_all_char(self):
        special_characters = self.SPECIAL_CHARACTERS.copy()
        for char in special_characters:
            if not char.isdigit():
                self.SPECIAL_CHARACTERS.remove(char)
        return super()._collect_all_char



import random


class CharacterSequence:
    """
    A class generating a random password of different characters.
    """
    SPECIAL_CHARACTERS = list('!@#$%^&*()_=+[]{};:\\"|,1234567890')
    CONTINUE_CHR = list(range(91, 97))
    ALPHABET_CHR = range(65, 123)

    def __init__(self):
        self.all_characters = []

    def __call__(self, length=6):
        return self.collect(length)

    @property
    def _collect_all_char(self):
        """
        Internal method returning a list of all characters.
        """
        for i in self.ALPHABET_CHR:
            if i in self.CONTINUE_CHR:
                continue
            self.all_characters.append(chr(i))
        self.all_characters.extend(self.SPECIAL_CHARACTERS)
        return self.all_characters

    def collect(self, length=6):
        """
        A collection that returns a string of random characters.
        :param length: 0 <= int <= 75
        :return: '' <- random string of characters
        """
        random.shuffle(self._collect_all_char)
        try:
            result = self._collect(length, self._collect_all_char)
        except ValueError:
            return None
        except TypeError:
            return None
        else:
            return result

    @staticmethod
    def _collect(length, collect_chars):
        length = int(length)
        if length == 6:
            random.shuffle(collect_chars)
            return ''.join(collect_chars[:6])
        elif 0 < length <= 75:
            random.shuffle(collect_chars)
            return ''.join(collect_chars[:length])
        else:
            raise ValueError('length can be from 1 to 75 characters')



import unittest

from char_collection.collect_password import CollectPassword


class MyCollectCharTest(unittest.TestCase):

    def test_normal(self):
        collect = CollectPassword()
        result = collect.collect(10)
        self.assertEqual(len(result), 10, 'Unexpected length returned (10)')

    def test_zero(self):
        collect = CollectPassword()
        result = collect.collect(0)
        self.assertEqual(result, None, 'in test_zero return not None')

    def test_big(self):
        collect = CollectPassword()
        result = collect.collect(999)
        self.assertEqual(result, None, 'in test big return not None')

    def test_negative(self):
        collect = CollectPassword()
        result = collect.collect(-1)
        self.assertEqual(result, None, 'in test negative return not None')

    def test_string(self):
        collect = CollectPassword()
        result = collect.collect('dsds')
        self.assertEqual(result, None, 'in test string return not None')

    def test_int_string(self):
        collect = CollectPassword()
        result = collect.collect('10')
        self.assertEqual(len(result), 10, 'in test int string return not None')


if __name__ == '__main__':
    unittest.main()
